// hdRiff.h: interface for the Riff/Wave/BWF classes.
//
//////////////////////////////////////////////////////////////////////

#ifndef __hdRIFF__
#define __hdRIFF__


#include <stdio.h>
#include <string.h>
#include <time.h>
#include "hdDiskMgr.h"

enum ChanTypes {
    ctSTEREO   = 0,
    ctMONOLEFT = 1,
    ctMONORIGHT= 2,
    ctMONOSUM  = 3,
    ctDUALMONO = 4
};

#pragma pack(1)

struct WaveSample
{
   INT32  chan [MAX_WAVE_CHANNELS];
};

struct RiffChunkHeader {
   FOURCC   ckID;       // Four-character chunk ID
   DWORD	ckSize;     // Length of data in chunk
};

// possible return values from RIFF file operations
enum RiffRet {
   RIFF_SUCCESS,
   RIFF_FAILURE,
   RIFF_OUT_OF_MEMORY,
   RIFF_FILE_ERROR,
   RIFF_INVALID_CALL,
   RIFF_USER_ABORT,
   RIFF_INVALID_FILE
};

enum RiffFileMode {
   RFM_UNKNOWN,      // undefined type (can use to mean "N/A" or "not open")
   RFM_WRITE,        // open for write
   RFM_READWRITE,    // BOTH
   RFM_READ,         // open for read
   RFM_PLAY,
   RFM_APPEND		 // append to an existing file
};



// ------------------------------------------------
// basic wave format data structure w/pcm addition (bitspersample)
//
struct WaveFormatChunkData
{
public:
	WaveFormatChunkData();
	~WaveFormatChunkData();

public:
	WORD		wFormatTag;
	WORD		nChannels;
	DWORD		nSamplesPerSec;
	DWORD		nAvgBytesPerSec;
	WORD		nBlockAlign;
	WORD		nBitsPerSample;
	WORD		cbSize;

	void Config (DWORD  rate = 44100,
				 WORD   bits = 16,
				 WORD   chan = 2 );
};


// ------------------------------------------------
// a 'PAD ' header for forcing file alignment (on sector boundary)

#define PAD_SIZE   70  
struct PadChunk
{
public:
	PadChunk() {
        header.ckID   = PADDING;
        header.ckSize = PAD_SIZE;
		memset(&data,'\0', PAD_SIZE);
    }
    ~PadChunk() { }
    
public:
	RiffChunkHeader     header;
	char				data[PAD_SIZE];
};


// ------------------------------------------------
// the overall wave 'fmt' component w/header info
struct WaveFormatChunk
{
public:
	WaveFormatChunk();
	~WaveFormatChunk();

public:
	RiffChunkHeader       header;
	WaveFormatChunkData    data;

	bool IsValid();
};


#define BASEID_NAME_SIZE   8
#define BASEID_DATE_SIZE  10
#define BASEID_TIME_SIZE   8

struct BaseIDChunk
{
public:
    BaseIDChunk();
    ~BaseIDChunk();

public:
   	RiffChunkHeader     header;
    char                baseIDName[BASEID_NAME_SIZE];
    char                baseIDDate[BASEID_DATE_SIZE];
    char                baseIDTime[BASEID_TIME_SIZE];
};


// -----------------------------------------------------------
// data of broadcast wave extension
struct BextChunkData {				// (all chars are ASCII
public:
	BextChunkData();
	~BextChunkData();

public:
	char	Description[256];			// desc of sound sequence
	char	Originator[32];				// Name of originator
	char	OriginatorReference[32];	// ref of ""  ""
	char	OriginationDate[10];		// yyyy-mm-dd
	char	OriginationTime[8];			// hh-mm-ss
	DWORD	TimeReferenceLow;			// first sample count since midnight, low WORD
	DWORD	TimeReferenceHi;			// first sample count since midnight, hi WORD
	WORD	Version;					// version of BWF, unsigned binary number
	char	UMIDS[64];					// binary bytes of SMPTE UMID
	char	Reserved[190];				// reserved for future use, set to NULL

	void	SetStartLocation(QWORD);

	void	SetOriginationTime(tm&);
	tm      GetOriginationTime();
	QWORD   GetStartLocation();

	void Config();
};


// ------------------------------------------------
// the overall BWF 'bext' component w/header info
struct BextFormatChunk
{
public:
	BextFormatChunk();
	~BextFormatChunk();

public:
	RiffChunkHeader     header;
	BextChunkData		data;
	char				codingHistory[256];				// History coding 

	bool IsValid();
	void SetHistory(char*,int);
};


// ------------------------------------------------
//
class hdRiff
{
public:

	hdRiff();
	~hdRiff();

public:
	RiffChunkHeader		riff_header;		// header for whole file

protected:
	RiffFileMode		fmode;				// current file I/O mode
    FILE*				fp;					// I/O stream to use

	RiffRet				Seek(QWORD newpos);
	RiffRet				Skip(QWORD offset);
	void				FlushFile() {if (fp) hdDiskMgr::Flush(fp); }
	RiffRet				ReadChunkHeader( RiffChunkHeader& riffCk );
	RiffRet				ReadWaveChunk( WaveFormatChunkData& waveCk, DWORD ckSize);
	RiffRet				ReadBextChunk( BextFormatChunk& bextCk, DWORD ckSize);
	RiffRet				ReadBaseIDChunk( BaseIDChunk& idCk);
    
public:
	RiffRet				SeekToEnd();

	RiffRet				CurrentFilePosition(QWORD&);
	RiffFileMode		CurrentFileMode() {return fmode;}

	RiffRet				Open(char*, RiffFileMode);
	RiffRet				Write(char*, QWORD, bool newdata = true);
	RiffRet				Read(char*, QWORD, QWORD& bytesRead);
	RiffRet				ReadDWORD(DWORD&);
	RiffRet				Close();

	static DWORD		FourCC(const char*);
};

#ifndef MAC_BUILD
#pragma pack()
#endif

#endif /* __hdRIFF__ */
